# wireless-sensor-mqtt - MQTT client reporting measurements of FT017TH wireless thermo/hygrometers
#
# Copyright (C) 2020 Fabian Peter Hammerle <fabian@hammerle.me>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import datetime
import json
import logging
import ssl
import typing
import unittest.mock

import _pytest
import pytest
import wireless_sensor

import wireless_sensor_mqtt

# pylint: disable=protected-access,too-many-positional-arguments


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "mqtt_topic_prefix", ["wireless-sensor/FT017TH", "living-room/ft017th"]
)
@pytest.mark.parametrize("homeassistant_discovery_prefix", ["homeassistant"])
@pytest.mark.parametrize(
    "homeassistant_node_id", ["ft017th-living-room", "bed-room-sensor"]
)
async def test__publish_homeassistant_discovery_config(
    mqtt_topic_prefix, homeassistant_discovery_prefix, homeassistant_node_id
):
    mqtt_client_mock = unittest.mock.AsyncMock()
    await wireless_sensor_mqtt._publish_homeassistant_discovery_config(
        mqtt_client=mqtt_client_mock,
        homeassistant_discovery_prefix=homeassistant_discovery_prefix,
        homeassistant_node_id=homeassistant_node_id,
        temperature_topic=mqtt_topic_prefix + "/temp",
        humidity_topic=mqtt_topic_prefix + "/rel-humidity",
    )
    publish_calls_args = mqtt_client_mock.publish.call_args_list
    assert len(publish_calls_args) == 2
    for call_args in publish_calls_args:
        assert not call_args[0]  # positional args
        assert set(call_args[1].keys()) == {"topic", "payload", "retain"}
        assert call_args[1]["retain"] is True
    assert (
        publish_calls_args[0][1]["topic"]
        == f"homeassistant/sensor/{homeassistant_node_id}/temperature-degrees-celsius/config"
    )
    assert (
        publish_calls_args[1][1]["topic"]
        == f"homeassistant/sensor/{homeassistant_node_id}/relative-humidity-percent/config"
    )
    device_attrs = {
        "identifiers": ["FT017TH/" + homeassistant_node_id],
        "model": "FT017TH",
    }
    assert json.loads(publish_calls_args[0][1]["payload"]) == {
        "unique_id": f"fphammerle/wireless-sensor-mqtt/FT017TH/{homeassistant_node_id}"
        "/temperature-degrees-celsius",
        "name": f"{homeassistant_node_id} temperature",
        "state_topic": mqtt_topic_prefix + "/temp",
        "device_class": "temperature",
        "unit_of_measurement": "°C",
        "expire_after": 600,
        "device": device_attrs,
    }
    assert json.loads(publish_calls_args[1][1]["payload"]) == {
        "unique_id": f"fphammerle/wireless-sensor-mqtt/FT017TH/{homeassistant_node_id}"
        "/relative-humidity-percent",
        "name": f"{homeassistant_node_id} relative humidity",
        "state_topic": mqtt_topic_prefix + "/rel-humidity",
        "device_class": "humidity",
        "unit_of_measurement": "%",
        "expire_after": 600,
        "device": device_attrs,
    }


async def _ft017th_receive_mock(
    sensor: wireless_sensor.FT017TH, timeout_seconds: int
) -> typing.AsyncIterator[wireless_sensor.Measurement]:
    assert isinstance(sensor, wireless_sensor.FT017TH)
    assert timeout_seconds == 60 * 60
    yield wireless_sensor.Measurement(
        decoding_timestamp=datetime.datetime(2020, 12, 7, 18, 5, 1),
        temperature_degrees_celsius=23.1234567,
        relative_humidity=0.501234567,
    )
    yield wireless_sensor.Measurement(
        decoding_timestamp=datetime.datetime(2020, 12, 7, 18, 6, 19),
        temperature_degrees_celsius=24.1234567,
        relative_humidity=0.401234567,
    )


@pytest.mark.asyncio
@pytest.mark.parametrize("mqtt_host", ["mqtt-broker.local"])
@pytest.mark.parametrize("mqtt_port", [1234])
@pytest.mark.parametrize("mqtt_disable_tls", [True, False])
@pytest.mark.parametrize(
    ("mqtt_username", "mqtt_password"),
    [(None, None), ("user", None), ("username", "secret")],
)
@pytest.mark.parametrize(
    "mqtt_topic_prefix", ["wireless-sensor/FT017TH", "living-room/ft017th"]
)
@pytest.mark.parametrize("homeassistant_discovery_prefix", ["homeassistant"])
@pytest.mark.parametrize("homeassistant_node_id", ["ft017th-living-room"])
@pytest.mark.parametrize("gdo0_gpio_line_name", [b"GPIO42"])
@pytest.mark.parametrize("unlock_spi_device", [True, False])
async def test__run(  # pylint: disable=too-many-locals
    caplog: _pytest.logging.LogCaptureFixture,
    mqtt_host,
    mqtt_port,
    mqtt_disable_tls,
    mqtt_username,
    mqtt_password,
    mqtt_topic_prefix,
    homeassistant_discovery_prefix,
    homeassistant_node_id,
    gdo0_gpio_line_name: bytes,
    unlock_spi_device,
):
    # pylint: disable=too-many-arguments
    with unittest.mock.patch(
        "wireless_sensor.FT017TH.__init__", return_value=None
    ) as sensor_init_mock, unittest.mock.patch(
        "wireless_sensor.FT017TH.receive", _ft017th_receive_mock
    ):
        with unittest.mock.patch(
            "aiomqtt.Client"
        ) as mqtt_client_class_mock, unittest.mock.patch(
            "wireless_sensor_mqtt._publish_homeassistant_discovery_config"
        ) as hass_config_mock, pytest.raises(
            RuntimeError, match=r"^timeout waiting for packet$"
        ):
            caplog.set_level(logging.DEBUG)
            await wireless_sensor_mqtt._run(
                mqtt_host=mqtt_host,
                mqtt_port=mqtt_port,
                mqtt_disable_tls=mqtt_disable_tls,
                mqtt_username=mqtt_username,
                mqtt_password=mqtt_password,
                mqtt_topic_prefix=mqtt_topic_prefix,
                homeassistant_discovery_prefix=homeassistant_discovery_prefix,
                homeassistant_node_id=homeassistant_node_id,
                mock_measurements=False,
                gdo0_gpio_line_name=gdo0_gpio_line_name,
                unlock_spi_device=unlock_spi_device,
            )
    mqtt_client_class_mock.assert_called_once()
    assert not mqtt_client_class_mock.call_args[0]  # args
    mqtt_client_init_kwargs = mqtt_client_class_mock.call_args[1]
    assert set(mqtt_client_init_kwargs.keys()) == {
        "tls_context",
        "hostname",
        "port",
        "username",
        "password",
    }
    if mqtt_disable_tls:
        assert mqtt_client_init_kwargs["tls_context"] is None
    else:
        assert isinstance(mqtt_client_init_kwargs["tls_context"], ssl.SSLContext)
    assert mqtt_client_init_kwargs["hostname"] == mqtt_host
    assert mqtt_client_init_kwargs["port"] == mqtt_port
    assert mqtt_client_init_kwargs["username"] == mqtt_username
    assert mqtt_client_init_kwargs["password"] == mqtt_password
    mqtt_client_class_mock.return_value.__aenter__.assert_called_once_with()
    assert caplog.records[0].levelno == logging.INFO
    assert caplog.records[0].message == (
        # pylint: disable=consider-using-f-string
        "connecting to MQTT broker {}:{} (TLS {})".format(
            mqtt_host, mqtt_port, "disabled" if mqtt_disable_tls else "enabled"
        )
    )
    assert caplog.records[1].levelno == logging.DEBUG
    assert caplog.records[1].message == (
        f"connected to MQTT broker {mqtt_host}:{mqtt_port}"
    )
    assert caplog.records[2].levelno == logging.DEBUG
    assert caplog.records[2].message == (
        "publishing measurements on topics"
        f" '{mqtt_topic_prefix}/temperature-degrees-celsius'"
        f" and '{mqtt_topic_prefix}/relative-humidity-percent'"
    )
    sensor_init_mock.assert_called_once_with(
        gdo0_gpio_line_name=gdo0_gpio_line_name, unlock_spi_device=unlock_spi_device
    )
    mqtt_client_instance_mock = (
        mqtt_client_class_mock.return_value.__aenter__.return_value
    )
    hass_config_mock.assert_called_once_with(
        mqtt_client=mqtt_client_instance_mock,
        homeassistant_discovery_prefix=homeassistant_discovery_prefix,
        homeassistant_node_id=homeassistant_node_id,
        temperature_topic=mqtt_topic_prefix + "/temperature-degrees-celsius",
        humidity_topic=mqtt_topic_prefix + "/relative-humidity-percent",
    )
    publish_calls_args = mqtt_client_instance_mock.publish.call_args_list
    assert len(publish_calls_args) == 2 * 2
    for call_args in publish_calls_args:
        assert not call_args[0]  # positional args
        assert set(call_args[1].keys()) == {"topic", "payload", "retain"}
        assert call_args[1]["retain"] is False
    assert (
        publish_calls_args[0][1]["topic"]
        == publish_calls_args[2][1]["topic"]
        == mqtt_topic_prefix + "/temperature-degrees-celsius"
    )
    assert (
        publish_calls_args[1][1]["topic"]
        == publish_calls_args[3][1]["topic"]
        == mqtt_topic_prefix + "/relative-humidity-percent"
    )
    assert publish_calls_args[0][1]["payload"] == "23.12"
    assert publish_calls_args[1][1]["payload"] == "50.12"
    assert publish_calls_args[2][1]["payload"] == "24.12"
    assert publish_calls_args[3][1]["payload"] == "40.12"
    assert caplog.records[3].levelno == logging.DEBUG
    assert caplog.records[3].message == (
        "received Measurement("
        "decoding_timestamp=datetime.datetime(2020, 12, 7, 18, 5, 1)"
        ", temperature_degrees_celsius=23.1234567"
        ", relative_humidity=0.501234567)"
    )
    assert caplog.records[4].levelno == logging.DEBUG
    assert caplog.records[4].message == (
        "received Measurement("
        "decoding_timestamp=datetime.datetime(2020, 12, 7, 18, 6, 19)"
        ", temperature_degrees_celsius=24.1234567"
        ", relative_humidity=0.401234567)"
    )
    assert not caplog.records[5:]


@pytest.mark.asyncio
@pytest.mark.parametrize("mqtt_disable_tls", [True, False])
async def test__run_mqtt_client_authentication_missing_username(mqtt_disable_tls):
    with unittest.mock.patch("aiomqtt.Client") as mqtt_client_class_mock, pytest.raises(
        ValueError, match=r"^Missing MQTT username$"
    ):
        await wireless_sensor_mqtt._run(
            mqtt_host="mqtt-broker.local",
            mqtt_port=1883,
            mqtt_disable_tls=mqtt_disable_tls,
            mqtt_username=None,
            mqtt_password="secret",
            mqtt_topic_prefix="wireless-sensor/FT017TH",
            homeassistant_discovery_prefix="homeassistant",
            homeassistant_node_id="ft017th-living-room",
            mock_measurements=False,
            gdo0_gpio_line_name=b"GPIO42",
            unlock_spi_device=False,
        )
    mqtt_client_class_mock.assert_not_called()


@pytest.mark.asyncio
@pytest.mark.parametrize("mqtt_topic_prefix", ["ft017th"])
async def test__run_mock_measurements(mqtt_topic_prefix):
    # pylint: disable=too-many-arguments
    with unittest.mock.patch(
        "aiomqtt.Client"
    ) as mqtt_client_class_mock, unittest.mock.patch(
        "time.sleep"
    ) as sleep_mock, unittest.mock.patch(
        "wireless_sensor_mqtt._publish_homeassistant_discovery_config"
    ) as hass_config_mock, pytest.raises(
        RuntimeError, match=r"^timeout waiting for packet$"
    ):
        await wireless_sensor_mqtt._run(
            mqtt_host="mqtt-broker.local",
            mqtt_port=1234,
            mqtt_disable_tls=True,
            mqtt_username=None,
            mqtt_password=None,
            mqtt_topic_prefix=mqtt_topic_prefix,
            homeassistant_discovery_prefix="homeassistant",
            homeassistant_node_id="ft017th-living-room",
            mock_measurements=True,
            gdo0_gpio_line_name=b"GPIO24",
            unlock_spi_device=False,
        )
    hass_config_mock.assert_called_once()
    assert all(c[0][0] == 8 for c in sleep_mock.call_args_list)
    mqtt_client_instance_mock = (
        mqtt_client_class_mock.return_value.__aenter__.return_value
    )
    publish_calls_args = mqtt_client_instance_mock.publish.call_args_list
    assert len(publish_calls_args) == 2 * 3
    for call_args in publish_calls_args:
        assert not call_args[0]  # positional args
        assert set(call_args[1].keys()) == {"topic", "payload", "retain"}
        assert call_args[1]["retain"] is False
        assert float(call_args[1]["payload"]) > 0
    assert (
        publish_calls_args[0][1]["topic"]
        == publish_calls_args[2][1]["topic"]
        == publish_calls_args[4][1]["topic"]
        == mqtt_topic_prefix + "/temperature-degrees-celsius"
    )
    assert (
        publish_calls_args[1][1]["topic"]
        == publish_calls_args[3][1]["topic"]
        == publish_calls_args[5][1]["topic"]
        == mqtt_topic_prefix + "/relative-humidity-percent"
    )



# wireless-sensor-mqtt - MQTT client reporting measurements of FT017TH wireless thermo/hygrometers
#
# Copyright (C) 2020 Fabian Peter Hammerle <fabian@hammerle.me>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import argparse
import asyncio
import json
import logging
import pathlib
import random
import ssl
import time
import typing

import aiomqtt
import wireless_sensor

import wireless_sensor_mqtt._homeassistant

_MQTT_DEFAULT_PORT = 1883
_MQTT_DEFAULT_TLS_PORT = 8883
_MEASUREMENT_MOCKS_COUNT = 3
_MEASUREMENT_MOCKS_INTERVAL_SECONDS = 8

_LOGGER = logging.getLogger(__name__)


def _mock_measurement() -> wireless_sensor.Measurement:
    time.sleep(_MEASUREMENT_MOCKS_INTERVAL_SECONDS)
    return wireless_sensor.Measurement(
        decoding_timestamp=None,
        temperature_degrees_celsius=random.uniform(20.0, 30.0),
        relative_humidity=random.uniform(0.4, 0.6),
    )


async def _publish_homeassistant_discovery_config(
    *,
    mqtt_client: aiomqtt.Client,
    homeassistant_discovery_prefix: str,
    homeassistant_node_id: str,
    temperature_topic: str,
    humidity_topic: str,
) -> None:
    """
    https://www.home-assistant.io/docs/mqtt/discovery/
    """
    # topic format: <discovery_prefix>/<component>/[<node_id>/]<object_id>/config
    # https://www.home-assistant.io/docs/mqtt/discovery/
    # https://github.com/home-assistant/core/blob/0.117.5/homeassistant/components/mqtt/__init__.py#L274
    device_attrs = {
        # > voluptuous.error.MultipleInvalid: Device must have at least one id
        # > entifying value in 'identifiers' and/or 'connections'
        # > for dictionary value @ data['device']
        "identifiers": [f"FT017TH/{homeassistant_node_id}"],
        "model": "FT017TH",
    }
    for object_id, device_class, state_topic, unit, name_suffix in zip(
        ("temperature-degrees-celsius", "relative-humidity-percent"),
        # https://www.home-assistant.io/integrations/sensor/#device-class
        ("temperature", "humidity"),
        (temperature_topic, humidity_topic),
        ("°C", "%"),
        ("temperature", "relative humidity"),
    ):
        discovery_topic = "/".join(
            (
                homeassistant_discovery_prefix,
                "sensor",
                homeassistant_node_id,
                object_id,
                "config",
            )
        )
        unique_id = "/".join(
            (
                "fphammerle",
                "wireless-sensor-mqtt",
                "FT017TH",
                homeassistant_node_id,
                object_id,
            )
        )
        # https://www.home-assistant.io/integrations/sensor.mqtt/#configuration-variables
        # https://github.com/home-assistant/core/blob/0.117.5/homeassistant/components/mqtt/sensor.py#L50
        config = {
            "unique_id": unique_id,
            "device_class": device_class,
            # friendly_name & template for default entity_id
            "name": f"{homeassistant_node_id} {name_suffix}",
            "state_topic": state_topic,
            "unit_of_measurement": unit,
            "expire_after": 60 * 10,  # seconds
            "device": device_attrs,
        }
        await mqtt_client.publish(
            topic=discovery_topic, payload=json.dumps(config), retain=True
        )


async def _measurement_iter(
    mock_measurements: bool, gdo0_gpio_line_name: bytes, unlock_spi_device: bool
) -> typing.AsyncIterator[wireless_sensor.Measurement]:
    if mock_measurements:
        logging.warning("publishing %d mocked measurements", _MEASUREMENT_MOCKS_COUNT)
        for _ in range(_MEASUREMENT_MOCKS_COUNT):
            yield _mock_measurement()
    else:
        async for measurement in wireless_sensor.FT017TH(
            gdo0_gpio_line_name=gdo0_gpio_line_name, unlock_spi_device=unlock_spi_device
        ).receive(timeout_seconds=3600):
            yield measurement


async def _run(  # pylint: disable=too-many-locals
    *,
    mqtt_host: str,
    mqtt_port: int,
    mqtt_disable_tls: bool,
    mqtt_username: typing.Optional[str],
    mqtt_password: typing.Optional[str],
    mqtt_topic_prefix: str,
    homeassistant_discovery_prefix: str,
    homeassistant_node_id: str,
    mock_measurements: bool,
    gdo0_gpio_line_name: bytes,
    unlock_spi_device: bool,
) -> None:
    # pylint: disable=too-many-arguments
    _LOGGER.info(
        "connecting to MQTT broker %s:%d (TLS %s)",
        mqtt_host,
        mqtt_port,
        "disabled" if mqtt_disable_tls else "enabled",
    )
    if mqtt_password and not mqtt_username:
        raise ValueError("Missing MQTT username")
    async with aiomqtt.Client(
        tls_context=None if mqtt_disable_tls else ssl.create_default_context(),
        hostname=mqtt_host,
        port=mqtt_port,
        username=mqtt_username,
        password=mqtt_password,
    ) as mqtt_client:
        _LOGGER.debug("connected to MQTT broker %s:%d", mqtt_host, mqtt_port)
        temperature_topic = mqtt_topic_prefix + "/temperature-degrees-celsius"
        humidity_topic = mqtt_topic_prefix + "/relative-humidity-percent"
        _LOGGER.debug(
            "publishing measurements on topics %r and %r",
            temperature_topic,
            humidity_topic,
        )
        homeassistant_discover_config_published = False
        async for measurement in _measurement_iter(
            mock_measurements=mock_measurements,
            gdo0_gpio_line_name=gdo0_gpio_line_name,
            unlock_spi_device=unlock_spi_device,
        ):
            _LOGGER.debug("received %s", measurement)
            if not homeassistant_discover_config_published:
                await _publish_homeassistant_discovery_config(
                    mqtt_client=mqtt_client,
                    homeassistant_discovery_prefix=homeassistant_discovery_prefix,
                    homeassistant_node_id=homeassistant_node_id,
                    temperature_topic=temperature_topic,
                    humidity_topic=humidity_topic,
                )
                homeassistant_discover_config_published = True
            await mqtt_client.publish(
                topic=temperature_topic,
                payload=f"{measurement.temperature_degrees_celsius:.02f}",
                retain=False,
            )
            await mqtt_client.publish(
                topic=humidity_topic,
                # > unit_of_measurement: '%'
                # https://www.home-assistant.io/integrations/sensor.mqtt/#temperature-and-humidity-sensors
                payload=f"{(measurement.relative_humidity * 100):.02f}",
                retain=False,
            )
        raise RuntimeError("timeout waiting for packet")


def _main() -> None:
    argparser = argparse.ArgumentParser(
        description="MQTT client reporting measurements of FT017TH wireless thermo/hygrometers",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    argparser.add_argument("--mqtt-host", type=str, required=True)
    argparser.add_argument(
        "--mqtt-port",
        type=int,
        help=f"default {_MQTT_DEFAULT_TLS_PORT} ({_MQTT_DEFAULT_PORT} with --mqtt-disable-tls)",
    )
    argparser.add_argument("--mqtt-username", type=str)
    argparser.add_argument("--mqtt-disable-tls", action="store_true")
    password_argument_group = argparser.add_mutually_exclusive_group()
    password_argument_group.add_argument("--mqtt-password", type=str)
    password_argument_group.add_argument(
        "--mqtt-password-file",
        type=pathlib.Path,
        metavar="PATH",
        dest="mqtt_password_path",
        help="stripping trailing newline",
    )
    argparser.add_argument(
        "--mqtt-topic-prefix",
        type=str,
        default="wireless-sensor/FT017TH",
        help=" ",  # show default
    )
    # https://www.home-assistant.io/docs/mqtt/discovery/#discovery_prefix
    argparser.add_argument(
        "--homeassistant-discovery-prefix", type=str, default="homeassistant", help=" "
    )
    argparser.add_argument(
        "--homeassistant-node-id",
        type=str,
        # pylint: disable=protected-access
        default="FT017TH",
        help=" ",
    )
    argparser.add_argument(
        "--mock-measurements",
        action="store_true",
        help="publish random values to test MQTT connection",
    )
    argparser.add_argument(
        "--gdo0-gpio-line-name",
        type=str,
        required=True,
        # GPIO24 recommended at
        # https://github.com/fphammerle/python-cc1101/tree/v2.7.3#wiring-raspberry-pi
        help="Name of GPIO pin that CC1101's GDO0 pin is connected to."
        " Run command `gpioinfo` to get a list of all available GPIO lines."
        " Recommended: GPIO24",
    )
    # https://github.com/fphammerle/wireless-sensor/blob/v0.3.0/wireless_sensor/_cli.py#L28
    argparser.add_argument(
        "--unlock-spi-device",
        action="store_true",
        help="Release flock from SPI device file after configuring the transceiver."
        " Useful if another process (infrequently) accesses the transceiver simultaneously.",
    )
    argparser.add_argument("--debug", action="store_true", help="increase verbosity")
    argparser.add_argument(
        "--debug-cc1101",
        action="store_true",
        help="increase verbosity of cc1101 library",
    )
    args = argparser.parse_args()
    logging.basicConfig(
        level=logging.DEBUG if args.debug else logging.INFO,
        format="%(asctime)s:%(levelname)s:%(message)s",
        datefmt="%Y-%m-%dT%H:%M:%S%z",
    )
    logging.getLogger("cc1101").setLevel(
        logging.DEBUG if args.debug_cc1101 else logging.INFO
    )
    if args.mqtt_port:
        mqtt_port = args.mqtt_port
    elif args.mqtt_disable_tls:
        mqtt_port = _MQTT_DEFAULT_PORT
    else:
        mqtt_port = _MQTT_DEFAULT_TLS_PORT
    if args.mqtt_password_path:
        # .read_text() replaces \r\n with \n
        mqtt_password = args.mqtt_password_path.read_bytes().decode()
        if mqtt_password.endswith("\r\n"):
            mqtt_password = mqtt_password[:-2]
        elif mqtt_password.endswith("\n"):
            mqtt_password = mqtt_password[:-1]
    else:
        mqtt_password = args.mqtt_password
    # pylint: disable=protected-access; false positive for validate_node_id
    if not wireless_sensor_mqtt._homeassistant.validate_node_id(
        args.homeassistant_node_id
    ):
        raise ValueError(
            f"invalid home assistant node id {args.homeassistant_node_id!r}"
            " (length >= 1, allowed characters:"
            f" {wireless_sensor_mqtt._homeassistant.NODE_ID_ALLOWED_CHARS})"
            "\nchange argument of --homeassistant-node-id"
        )
    asyncio.run(
        _run(
            mqtt_host=args.mqtt_host,
            mqtt_port=mqtt_port,
            mqtt_disable_tls=args.mqtt_disable_tls,
            mqtt_username=args.mqtt_username,
            mqtt_password=mqtt_password,
            mqtt_topic_prefix=args.mqtt_topic_prefix,
            homeassistant_discovery_prefix=args.homeassistant_discovery_prefix,
            homeassistant_node_id=args.homeassistant_node_id,
            mock_measurements=args.mock_measurements,
            gdo0_gpio_line_name=args.gdo0_gpio_line_name.encode(),
            unlock_spi_device=args.unlock_spi_device,
        )
    )



# wireless-sensor-mqtt - MQTT client reporting measurements of FT017TH wireless thermo/hygrometers
#
# Copyright (C) 2020 Fabian Peter Hammerle <fabian@hammerle.me>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import pathlib

import setuptools

_REPO_URL = "https://github.com/fphammerle/wireless-sensor-mqtt"

setuptools.setup(
    name="wireless-sensor-mqtt",
    use_scm_version={
        "git_describe_command": "git describe --dirty --tags --long --match v*"
    },
    packages=setuptools.find_packages(),
    description="MQTT client reporting measurements of FT017TH wireless thermo/hygrometers",
    long_description=pathlib.Path(__file__).parent.joinpath("README.md").read_text(),
    long_description_content_type="text/markdown",
    author="Fabian Peter Hammerle",
    author_email="fabian@hammerle.me",
    url=_REPO_URL,
    project_urls={"Changelog": _REPO_URL + "/blob/master/CHANGELOG.md"},
    license="GPLv3+",
    keywords=[
        "FT017TH",
        "IoT",
        "automation",
        "climate",
        "home-assistant",
        "home-automation",
        "humidity",
        "hygrometer",
        "mqtt",
        "raspberry-pi",
        "sensor",
        "thermometer",
        "wireless",
    ],
    classifiers=[
        # https://pypi.org/classifiers/
        "Development Status :: 3 - Alpha",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: POSIX :: Linux",
        # .github/workflows/python.yml
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Home Automation",
        "Topic :: Utilities",
    ],
    entry_points={
        "console_scripts": ["wireless-sensor-mqtt = wireless_sensor_mqtt:_main"]
    },
    python_requires=">=3.9",  # <3.9 untested
    install_requires=[
        # >=0.3.0 for FT017TH(unlock_spi_device=True)
        # >=0.4.0 for FT017TH(gdo0_gpio_line_name=…)
        # >=1.0.0 for async FT017TH.receive()
        "wireless-sensor>=1.0.0,<2",
        "aiomqtt>=2.3,<3",  # <2.3 untested
    ],
    setup_requires=["setuptools_scm"],
    tests_require=["pytest"],
)
